/* cris-opc.c -- Table of opcodes for the CRIS processor.
   Copyright (C) 1989, 1999 Free Software Foundation, Inc.

This file is part of GAS, GDB and the GNU binutils.

GAS/GDB/GNU binutils is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GAS/GDB/GNU binutils is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GAS or GDB or GNU binutils; see the file COPYING.  If not, write to
the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */

#include "opcode/cris.h"

#ifndef NULL
#define NULL (0)
#endif

const struct cris_spec_reg cris_spec_regs[] =
{
  { "p0",  0,  1, 0,		    NULL },
  { "vr",  1,  1, 0,		    NULL },
  { "p1",  1,  1, 0,		    NULL },
  { "p2",  2,  1, cris_ver_warning, NULL },
  { "p3",  3,  1, cris_ver_warning, NULL },
  { "p4",  4,  2, 0,		    NULL },
  { "ccr", 5,  2, 0,		    NULL },
  { "p5",  5,  2, 0,		    NULL },
  { "dcr0",6,  2, cris_ver_v0_3,    NULL },
  { "p6",  6,  2, cris_ver_v0_3,    NULL },
  { "dcr1/mof", 7, 4, cris_ver_v10p,
    "Register `dcr1/mof' with ambiguous size specified.  Guessing 4 bytes" },
  { "dcr1/mof", 7, 2, cris_ver_v0_3,
    "Register `dcr1/mof' with ambiguous size specified.  Guessing 2 bytes" },
  { "mof", 7,  4, cris_ver_v10p,    NULL },
  { "dcr1",7,  2, cris_ver_v0_3,    NULL },
  { "p7",  7,  4, cris_ver_v10p,    NULL },
  { "p7",  7,  2, cris_ver_v0_3,    NULL },
  { "p8",  8,  4, 0,		    NULL },
  { "ibr", 9,  4, 0,		    NULL },
  { "p9",  9,  4, 0,		    NULL },
  { "irp", 10, 4, 0,		    NULL },
  { "p10", 10, 4, 0,		    NULL },
  { "srp", 11, 4, 0,		    NULL },
  { "p11", 11, 4, 0,		    NULL },
  { "bar/dtp0", 12, 4, cris_ver_warning,
    "Ambiguous register `bar/dtp0' specified" }, /* For disassembly use. */
  { "bar", 12, 4, cris_ver_v8p,	    NULL },
  { "dtp0",12, 4, cris_ver_v0_3,    NULL },
  { "p12", 12, 4, 0,		    NULL },
  { "dccr/dtp1",13, 4, cris_ver_warning,
    "Ambiguous register `dccr/dtp1' specified" }, /* For disassembly use */
  { "dccr",13, 4, cris_ver_v8p,	    NULL },
  { "dtp1",13, 4, cris_ver_v0_3,    NULL },
  { "p13", 13, 4, 0,		    NULL },
  { "brp", 14, 4, cris_ver_v3p,	    NULL },
  { "p14", 14, 4, cris_ver_v3p,	    NULL },
  { "usp", 15, 4, cris_ver_v10p,    NULL },
  { "p15", 15, 4, cris_ver_v10p,    NULL },
  { NULL, 0, 0, NULL }		   /* Terminate. */
};

/*
   All cris opcodes are 16 bits.

   The match component is a mask saying which bits must match a
   particular opcode in order for an instruction to be an instance
   of that opcode.

   The args component is a string containing one character
   for each operand of the instruction.

Kinds of operands:
   B    Not really an operand; outputs a "BDAP -size,SP" prefix for
	the PUSH instructions and recognizes a push-prefix in
	disassembly.  Must be followed by a R or P letter (used
	in disassembly).
   !	Non-match pattern, will not match if there's a prefix insn.
   b    Non-matching operand, used for branch with 16-bit displacement.
        Only recognized in the disassembler.
   c    5-bit unsigned immediate in bits <4:0>
   C    4-bit unsigned immediate in bits <3:0>
   D    General register in bits <15:12> and <3:0>
   f    List of flags in bits <15:12> and <3:0>
   i    6-bit signed immediate in bits <5:0>
   I    6-bit unsigned immediate in bits <5:0>
   M    Size modifier (B, W or D) for CLEAR instructions
   m    Size modifier (B, W or D) in bits <5:4>
   o    [-128..127] word offset in bits <7:1> and <0>
   O    [-128..127] offset in bits <7:0> comma and general reg: n,r
   P    Special register in bits <15:12>
   p    Changed: tells that insn is a prefix insn. Must be 1st char HP 930416
   R    General register in bits <15:12>
   r    General register in bits <3:0>
   S    Source operand in bit <10> and a prefix; a 3-operand prefix
   s    Source operand in bits <10> and <3:0> and optionally a prefix;
           no prefix or an effective-address prefix
   x    Rs.m in bits <15:12> and <5:4>
   y    Like 's' but do not allow an integer (at assembly).
   z    Size modifier (B or W) in bit <4>

*/


/* The order of the opcodes in this table is significant:
   
   * The assembler requires that all instances of the same mnemonic must be
     consecutive.  If they aren't, the assembler will bomb at runtime.

   * The disassembler should not care about the order of the opcodes.

   * The 'S' and 's' operand descriptors are syntactically overlapping -
     offset mode without an offset expression and indirect mode.

   * Register operands may be interpreted as immediate mode operands.  */


/* 920401  MA  First version completed                            */
/* 920508  MA  Updated to Preliminary specification - CGARISC     */
/*               INTRUCTION SET Rev. 0.12                         */
/* 920603  MA  Added imm_oprnd_size values                        */
/* 920604  MA  Added spaces after 'z' argument descriptors        */
/* 920626  MA  Added the PUSH and POP instructions                */
/* 920630  MA  Added NOP instruction, changed order of operands   */
/*               and added size modifier to the MUL and DIV       */
/*               instructions.                                    */
/* 920701  MA  Set the "delayed" field for delayed branch         */
/*               instructions.                                    */
/* 920713  MA  Use opcode constants in table for increased safety */
/* 920714  MA  Corrected test instruction operands                */
/* 920717  MA  Added the AX, CLEAR, DI and EI instructions        */
/* 930408  HP  Changed to comply with cris instruction set
                                               ver 0.31           */
/* 960802  HP  Added new Etrax 4 instructions: retb, lz, break,   */
/*             and related modifications; brp register            */
/* 961113  HP  Added svinto (Etrax V8) instructions               */
/* 970624  HP  Modified generated "jump"-opcode to using special  */
/*             register 0 as destination (was 8).  For reasons of */
/*             disassembly (etc), special register 8 is still     */
/*             recognized for jump.                               */
/* Do not add more revision notes here; use the ChangeLog. */
const struct cris_opcode cris_opcodes[] =
{
  { "abs",     0x06B0, 0x0940,             "r,R",     0, SIZE_NONE,   0 },
  { "add",     0x0600, 0x09c0,             "m r,R",   0, SIZE_NONE,   0 },
  { "add",     0x0A00, 0x01c0,             "m s,R",   0, SIZE_FIELD,  0 },
  { "add",     0x0A00, 0x01c0,             "m S,D",   0, SIZE_NONE,   0 },
  { "add",     0x0a00, 0x05c0,             "m S,R,r", 0, SIZE_NONE,   0 },
  { "addi",    0x0500, 0x0Ac0,             "x,r",     0, SIZE_NONE,   0 },
  { "addq",    0x0200, 0x0Dc0,             "I,R",     0, SIZE_NONE,   0 },
  { "adds",    0x0420, 0x0Bc0,             "z r,R",   0, SIZE_NONE,   0 },
  { "adds",    0x0820, 0x03c0,             "z s,R",   0, SIZE_FIELD,  0 },
  { "adds",    0x0820, 0x03c0,             "z S,D",   0, SIZE_NONE,   0 },
  { "adds",    0x0820, 0x07c0,             "z S,R,r", 0, SIZE_NONE,   0 },
  { "addu",    0x0400, 0x0be0,             "z r,R",   0, SIZE_NONE,   0 },
  { "addu",    0x0800, 0x03e0,             "z s,R",   0, SIZE_FIELD,  0 },
  { "addu",    0x0800, 0x03e0,             "z S,D",   0, SIZE_NONE,   0 },
  { "addu",    0x0800, 0x07e0,             "z S,R,r", 0, SIZE_NONE,   0 },
  { "and",     0x0700, 0x08C0,             "m r,R",   0, SIZE_NONE,   0 },
  { "and",     0x0B00, 0x00C0,             "m s,R",   0, SIZE_FIELD,  0 },
  { "and",     0x0B00, 0x00C0,             "m S,D",   0, SIZE_NONE,   0 },
  { "and",     0x0B00, 0x04C0,             "m S,R,r", 0, SIZE_NONE,   0 },
  { "andq",    0x0300, 0x0CC0,             "i,R",     0, SIZE_NONE,   0 },
  { "asr",     0x0780, 0x0840,             "m r,R",   0, SIZE_NONE,   0 },
  { "asrq",    0x03a0, 0x0c40,             "c,R",     0, SIZE_NONE,   0 },
  { "ax",      0x15B0, 0xEA4F,             "",        0, SIZE_NONE,   0 },
  /* Should use branch #defines */
  { "b",       0x0dff, 0x0200,             "b",       0, SIZE_NONE,   0 },
  { "ba",      BA_QUICK_OPCODE,
               0x0F00+(0xF-CC_A)*0x1000,   "o",       1, SIZE_NONE,   0 },
  { "bcc",     BRANCH_QUICK_OPCODE+CC_CC*0x1000,
               0x0f00+(0xF-CC_CC)*0x1000,  "o",       1, SIZE_NONE,   0 },
  { "bcs",     BRANCH_QUICK_OPCODE+CC_CS*0x1000,
               0x0f00+(0xF-CC_CS)*0x1000,  "o",       1, SIZE_NONE,   0 },
  { "bdap",    BDAP_INDIR_OPCODE,
               BDAP_INDIR_Z_BITS,          "pm s,R",  0, SIZE_FIELD,  0 },
  { "bdap",    BDAP_QUICK_OPCODE,
               BDAP_QUICK_Z_BITS,          "pO",      0, SIZE_NONE,   0 },
  { "beq",     BRANCH_QUICK_OPCODE+CC_EQ*0x1000,
               0x0f00+(0xF-CC_EQ)*0x1000,  "o",       1, SIZE_NONE,   0 },
  /* This is deliberately put before "bext" to trump it, albeit not in
     alphabetical order.  */
  { "bwf",     BRANCH_QUICK_OPCODE+CC_EXT*0x1000,
               0x0f00+(0xF-CC_EXT)*0x1000, "o",       1, SIZE_NONE,   cris_ver_v10p },
  { "bext",    BRANCH_QUICK_OPCODE+CC_EXT*0x1000,
               0x0f00+(0xF-CC_EXT)*0x1000, "o",       1, SIZE_NONE,   cris_ver_v0_3 },
  { "bge",     BRANCH_QUICK_OPCODE+CC_GE*0x1000,
               0x0f00+(0xF-CC_GE)*0x1000,  "o",       1, SIZE_NONE,   0 },
  { "bgt",     BRANCH_QUICK_OPCODE+CC_GT*0x1000,
               0x0f00+(0xF-CC_GT)*0x1000,  "o",       1, SIZE_NONE,   0 },
  { "bhi",     BRANCH_QUICK_OPCODE+CC_HI*0x1000,
               0x0f00+(0xF-CC_HI)*0x1000,  "o",       1, SIZE_NONE,   0 },
  { "bhs",     BRANCH_QUICK_OPCODE+CC_HS*0x1000,
               0x0f00+(0xF-CC_HS)*0x1000,  "o",       1, SIZE_NONE,   0 },
  { "biap",    BIAP_OPCODE, BIAP_Z_BITS,   "pm r,R",  0, SIZE_NONE,   0 },
  { "ble",     BRANCH_QUICK_OPCODE+CC_LE*0x1000,
               0x0f00+(0xF-CC_LE)*0x1000,  "o",       1, SIZE_NONE,   0 },
  { "blo",     BRANCH_QUICK_OPCODE+CC_LO*0x1000,
               0x0f00+(0xF-CC_LO)*0x1000,  "o",       1, SIZE_NONE,   0 },
  { "bls",     BRANCH_QUICK_OPCODE+CC_LS*0x1000,
               0x0f00+(0xF-CC_LS)*0x1000,  "o",       1, SIZE_NONE,   0 },
  { "blt",     BRANCH_QUICK_OPCODE+CC_LT*0x1000,
               0x0f00+(0xF-CC_LT)*0x1000,  "o",       1, SIZE_NONE,   0 },
  { "bmi",     BRANCH_QUICK_OPCODE+CC_MI*0x1000,
               0x0f00+(0xF-CC_MI)*0x1000,  "o",       1, SIZE_NONE,   0 },
  { "bmod",    0x0ab0, 0x0140,             "s,R",     0, SIZE_FIX_32, cris_ver_sim },
  { "bmod",    0x0ab0, 0x0140,             "S,D",     0, SIZE_NONE,   cris_ver_sim },
  { "bmod",    0x0ab0, 0x0540,             "S,R,r",   0, SIZE_NONE,   cris_ver_sim },
  { "bne",     BRANCH_QUICK_OPCODE+CC_NE*0x1000,
               0x0f00+(0xF-CC_NE)*0x1000,  "o",       1, SIZE_NONE,   0 },
  { "bound",   0x05c0, 0x0A00,             "m r,R",   0, SIZE_NONE,   0 },
  { "bound",   0x09c0, 0x0200,             "m s,R",   0, SIZE_FIELD,  0 },
  { "bound",   0x09c0, 0x0200,             "m S,D",   0, SIZE_NONE,   0 },
  { "bound",   0x09c0, 0x0600,             "m S,R,r", 0, SIZE_NONE,   0 },
  { "bpl",     BRANCH_QUICK_OPCODE+CC_PL*0x1000,
               0x0f00+(0xF-CC_PL)*0x1000,  "o",       1, SIZE_NONE,   0 },
  { "break",   0xe930, 0x16c0,             "C",       0, SIZE_NONE,   cris_ver_v3p },
  { "bstore",  0x0af0, 0x0100,             "s,R",     0, SIZE_FIX_32, cris_ver_warning },
  { "bstore",  0x0af0, 0x0100,             "S,D",     0, SIZE_NONE,   cris_ver_warning },
  { "bstore",  0x0af0, 0x0500,             "S,R,r",   0, SIZE_NONE,   cris_ver_warning },
  { "btst",    0x04F0, 0x0B00,             "r,R",     0, SIZE_NONE,   0 },
  { "btstq",   0x0380, 0x0C60,             "c,R",     0, SIZE_NONE,   0 },
  { "bvc",     BRANCH_QUICK_OPCODE+CC_VC*0x1000,
               0x0f00+(0xF-CC_VC)*0x1000,  "o",       1, SIZE_NONE,   0 },
  { "bvs",     BRANCH_QUICK_OPCODE+CC_VS*0x1000,
               0x0f00+(0xF-CC_VS)*0x1000,  "o",       1, SIZE_NONE,   0 },
  { "clear",   0x0670, 0x3980,             "M r",     0, SIZE_NONE,   0 },
  { "clear",   0x0A70, 0x3180,             "M y",     0, SIZE_NONE,   0 },
  { "clear",   0x0A70, 0x3180,             "M S",     0, SIZE_NONE,   0 },
  { "clearf",  0x05F0, 0x0A00,             "f",       0, SIZE_NONE,   0 },
  { "cmp",     0x06C0, 0x0900,             "m r,R",   0, SIZE_NONE,   0 },
  { "cmp",     0x0Ac0, 0x0100,             "m s,R",   0, SIZE_FIELD,  0 },
  { "cmp",     0x0Ac0, 0x0100,             "m S,D",   0, SIZE_NONE,   0 },
  { "cmpq",    0x02C0, 0x0D00,             "i,R",     0, SIZE_NONE,   0 },
  { "cmps",    0x08e0, 0x0300,             "z s,R",   0, SIZE_FIELD,  0 },
  { "cmps",    0x08e0, 0x0300,             "z S,D",   0, SIZE_NONE,   0 },
  { "cmpu",    0x08c0, 0x0320,             "z s,R" ,  0, SIZE_FIELD,  0 },
  { "cmpu",    0x08c0, 0x0320,             "z S,D",   0, SIZE_NONE,   0 },
  { "di",      0x25F0, 0xDA0F,             "",        0, SIZE_NONE,   0 },
  { "dip",     DIP_OPCODE, DIP_Z_BITS,     "ps",      0, SIZE_FIX_32, 0 },
  { "div",     0x0980, 0x0640,             "m R,r",   0, SIZE_FIELD,  0 },
  { "dstep",   0x06f0, 0x0900,             "r,R",     0, SIZE_NONE,   0 },
  { "ei",      0x25B0, 0xDA4F,             "",        0, SIZE_NONE,   0 },
  { "jbrc",    0x69b0, 0x9640,             "r",       0, SIZE_NONE,   cris_ver_v8p },
  { "jbrc",    0x6930, 0x92c0,             "s",       0, SIZE_FIX_32, cris_ver_v8p },
  { "jbrc",    0x6930, 0x92c0,             "S",       0, SIZE_NONE,   cris_ver_v8p },
  { "jir",     0xA9b0, 0x5640,             "r",       0, SIZE_NONE,   0 },
  { "jir",     0xA930, 0x52c0,             "s",       0, SIZE_FIX_32, 0 },
  { "jir",     0xA930, 0x52c0,             "S",       0, SIZE_NONE,   0 },
  { "jirc",    0x29b0, 0xd640,             "r",       0, SIZE_NONE,   cris_ver_v8p },
  { "jirc",    0x2930, 0xd2c0,             "s",       0, SIZE_FIX_32, cris_ver_v8p },
  { "jirc",    0x2930, 0xd2c0,             "S",       0, SIZE_NONE,   cris_ver_v8p },
  { "jsr",     0xB9b0, 0x4640,             "r",       0, SIZE_NONE,   0 },
  { "jsr",     0xB930, 0x42c0,             "s",       0, SIZE_FIX_32, 0 },
  { "jsr",     0xB930, 0x42c0,             "S",       0, SIZE_NONE,   0 },
  { "jsrc",    0x39b0, 0xc640,             "r",       0, SIZE_NONE,   cris_ver_v8p },
  { "jsrc",    0x3930, 0xc2c0,             "s",       0, SIZE_FIX_32, cris_ver_v8p },
  { "jsrc",    0x3930, 0xc2c0,             "S",       0, SIZE_NONE,   cris_ver_v8p },
  /* Note that both special register 8 and 0 (both are zero-valued) are
     recognized as jump-instructions, but special-register 0 is generated.
     /H-P 970624.  This is now only true for a register operand, and may
     change.  */
  { "jump",    0x09b0, 0x7640,             "r",       0, SIZE_NONE,   0 },
  { "jump",    JUMP_INDIR_OPCODE,
               JUMP_INDIR_Z_BITS,          "s",       0, SIZE_FIX_32, 0 },
  { "jump",    JUMP_INDIR_OPCODE,
               JUMP_INDIR_Z_BITS,          "S",       0, SIZE_NONE,   0 },
  { "jmpu",    0x8930, 0x72c0,		   "s",       0, SIZE_FIX_32, cris_ver_v10p },
  { "jmpu",    0x8930, 0x72c0,		   "S",       0, SIZE_NONE,   cris_ver_v10p },
  /* There used to be a line here for the originally specified but never
     implemented "link" insn.  It now collides when source reg is r3
     with "sbfs", therefore it was removed. */
  { "lsl",     0x04C0, 0x0B00,             "m r,R",   0, SIZE_NONE,   0 },
  { "lslq",    0x03c0, 0x0C20,             "c,R",     0, SIZE_NONE,   0 },
  { "lsr",     0x07C0, 0x0800,             "m r,R",   0, SIZE_NONE,   0 },
  { "lsrq",    0x03e0, 0x0C00,             "c,R",     0, SIZE_NONE,   0 },
  { "lz",      0x0730, 0x08C0,             "r,R",     0, SIZE_NONE,   cris_ver_v3p },
  { "move",    0x0640, 0x0980,             "m r,R",   0, SIZE_NONE,   0 },
  { "move",    0x0630, 0x09c0,             "r,P",     0, SIZE_NONE,   0 },
  { "move",    0x0670, 0x0980,             "P,r",     0, SIZE_NONE,   0 },
  { "move",    0x0BC0, 0x0000,             "m R,y",   0, SIZE_FIELD,  0 },
  { "move",    0x0BC0, 0x0000,             "m D,S",   0, SIZE_NONE,   0 },
  { "move",    0x0A40, 0x0180,             "m s,R",   0, SIZE_FIELD,  0 },
  { "move",    0x0A40, 0x0180,             "m S,D",   0, SIZE_NONE,   0 },
  { "move",    0x0A30, 0x01c0,             "s,P",     0, SIZE_SPEC_REG, 0 },
  { "move",    0x0A30, 0x01c0,             "S,P",     0, SIZE_NONE,   0 },
  { "move",    0x0A70, 0x0180,             "P,y",     0, SIZE_SPEC_REG, 0 },
  { "move",    0x0A70, 0x0180,             "P,S",     0, SIZE_NONE,   0 },
  { "movem",   0x0BF0, 0x0000,             "R,y",     0, SIZE_FIX_32, 0 },
  { "movem",   0x0BF0, 0x0000,             "D,S",     0, SIZE_NONE,   0 },
  { "movem",   0x0BB0, 0x0040,             "s,R",     0, SIZE_FIX_32, 0 },
  { "movem",   0x0BB0, 0x0040,             "S,D",     0, SIZE_NONE,   0 },
  { "moveq",   0x0240, 0x0D80,             "i,R",     0, SIZE_NONE,   0 },
  { "movs",    0x0460, 0x0B80,             "z r,R",   0, SIZE_NONE,   0 },
  { "movs",    0x0860, 0x0380,             "z s,R",   0, SIZE_FIELD,  0 },
  { "movs",    0x0860, 0x0380,             "z S,D",   0, SIZE_NONE,   0 },
  { "movu",    0x0440, 0x0Ba0,             "z r,R",   0, SIZE_NONE,   0 },
  { "movu",    0x0840, 0x03a0,             "z s,R",   0, SIZE_FIELD,  0 },
  { "movu",    0x0840, 0x03a0,             "z S,D",   0, SIZE_NONE,   0 },
  { "mstep",   0x07f0, 0x0800,             "r,R",     0, SIZE_NONE,   0 },
  { "muls",    0x0d00, 0x02c0,             "m r,R",   0, SIZE_NONE,   cris_ver_v10p },
  { "mulu",    0x0900, 0x06c0,             "m r,R",   0, SIZE_NONE,   cris_ver_v10p },
  { "neg",     0x0580, 0x0A40,             "m r,R",   0, SIZE_NONE,   0 },
  { "nop",     NOP_OPCODE, NOP_Z_BITS,     "",        0, SIZE_NONE,   0 },
  { "not",     0x8770, 0x7880,             "r",       0, SIZE_NONE,   0 },
  { "or",      0x0740, 0x0880,             "m r,R",   0, SIZE_NONE,   0 },
  { "or",      0x0B40, 0x0080,             "m s,R",   0, SIZE_FIELD,  0 },
  { "or",      0x0B40, 0x0080,             "m S,D",   0, SIZE_NONE,   0 },
  { "or",      0x0B40, 0x0480,             "m S,R,r", 0, SIZE_NONE,   0 },
  { "orq",     0x0340, 0x0C80,             "i,R",     0, SIZE_NONE,   0 },
  { "pop",     0x0E6E, 0x0191,		   "!R",      0, SIZE_NONE,   0 },
  { "pop",     0x0e3e, 0x01c1,		   "!P",      0, SIZE_NONE,   0 },
  { "push",    0x0FEE, 0x0011,             "BR",      0, SIZE_NONE,   0 },
  { "push",    0x0E7E, 0x0181,             "BP",      0, SIZE_NONE,   0 },
  { "rbf",     0x3b30, 0xc0c0,		   "y",	      0, SIZE_NONE,   cris_ver_v10p },
  { "rbf",     0x3b30, 0xc0c0,		   "S",	      0, SIZE_NONE,   cris_ver_v10p },
  { "ret",     0xB67F, 0x4980,             "",        1, SIZE_NONE,   0 },
  { "retb",    0xe67f, 0x1980,             "",        1, SIZE_NONE,   0 },
  { "reti",    0xA67F, 0x5980,             "",        1, SIZE_NONE,   0 },
  { "sbfs",    0x3b70, 0xc080,             "y",       0, SIZE_NONE,   cris_ver_v10p },
  { "sbfs",    0x3b70, 0xc080,             "S",       0, SIZE_NONE,   cris_ver_v10p },
  { "sa",      0x0530+CC_A*0x1000,
               0x0AC0+(0xf-CC_A)*0x1000,   "r",       0, SIZE_NONE,   0 },
  { "scc",     0x0530+CC_CC*0x1000,
               0x0AC0+(0xf-CC_CC)*0x1000,  "r",       0, SIZE_NONE,   0 },
  { "scs",     0x0530+CC_CS*0x1000,
               0x0AC0+(0xf-CC_CS)*0x1000,  "r",       0, SIZE_NONE,   0 },
  { "seq",     0x0530+CC_EQ*0x1000,
               0x0AC0+(0xf-CC_EQ)*0x1000,  "r",       0, SIZE_NONE,   0 },
  { "setf",    0x05b0, 0x0A40,             "f",       0, SIZE_NONE,   0 },

  /* Need to have "swf" in front of "sext" so it is the one displayed in
     disassembly.  */
  { "swf",     0x0530+CC_EXT*0x1000,
               0x0AC0+(0xf-CC_EXT)*0x1000, "r",       0, SIZE_NONE,   cris_ver_v10p },
  { "sext",    0x0530+CC_EXT*0x1000,
               0x0AC0+(0xf-CC_EXT)*0x1000, "r",       0, SIZE_NONE,   cris_ver_v0_3 },
  { "sge",     0x0530+CC_GE*0x1000,
               0x0AC0+(0xf-CC_GE)*0x1000,  "r",       0, SIZE_NONE,   0 },
  { "sgt",     0x0530+CC_GT*0x1000,
               0x0AC0+(0xf-CC_GT)*0x1000,  "r",       0, SIZE_NONE,   0 },
  { "shi",     0x0530+CC_HI*0x1000,
               0x0AC0+(0xf-CC_HI)*0x1000,  "r",       0, SIZE_NONE,   0 },
  { "shs",     0x0530+CC_HS*0x1000,
               0x0AC0+(0xf-CC_HS)*0x1000,  "r",       0, SIZE_NONE,   0 },
  { "sle",     0x0530+CC_LE*0x1000,
               0x0AC0+(0xf-CC_LE)*0x1000,  "r",       0, SIZE_NONE,   0 },
  { "slo",     0x0530+CC_LO*0x1000,
               0x0AC0+(0xf-CC_LO)*0x1000,  "r",       0, SIZE_NONE,   0 },
  { "sls",     0x0530+CC_LS*0x1000,
               0x0AC0+(0xf-CC_LS)*0x1000,  "r",       0, SIZE_NONE,   0 },
  { "slt",     0x0530+CC_LT*0x1000,
               0x0AC0+(0xf-CC_LT)*0x1000,  "r",       0, SIZE_NONE,   0 },
  { "smi",     0x0530+CC_MI*0x1000,
               0x0AC0+(0xf-CC_MI)*0x1000,  "r",       0, SIZE_NONE,   0 },
  { "sne",     0x0530+CC_NE*0x1000,
               0x0AC0+(0xf-CC_NE)*0x1000,  "r",       0, SIZE_NONE,   0 },
  { "spl",     0x0530+CC_PL*0x1000,
               0x0AC0+(0xf-CC_PL)*0x1000,  "r",       0, SIZE_NONE,   0 },
  { "sub",     0x0680, 0x0940,             "m r,R",   0, SIZE_NONE,   0 },
  { "sub",     0x0a80, 0x0140,             "m s,R",   0, SIZE_FIELD,  0 },
  { "sub",     0x0a80, 0x0140,             "m S,D",   0, SIZE_NONE,   0 },
  { "sub",     0x0a80, 0x0540,             "m S,R,r", 0, SIZE_NONE,   0 },
  { "subq",    0x0280, 0x0d40,             "I,R",     0, SIZE_NONE,   0 },
  { "subs",    0x04a0, 0x0b40,             "z r,R",   0, SIZE_NONE,   0 },
  { "subs",    0x08a0, 0x0340,             "z s,R",   0, SIZE_FIELD,  0 },
  { "subs",    0x08a0, 0x0340,             "z S,D",   0, SIZE_NONE,   0 },
  { "subs",    0x08a0, 0x0740,             "z S,R,r", 0, SIZE_NONE,   0 },
  { "subu",    0x0480, 0x0b60,             "z r,R",   0, SIZE_NONE,   0 },
  { "subu",    0x0880, 0x0360,             "z s,R",   0, SIZE_FIELD,  0 },
  { "subu",    0x0880, 0x0360,             "z S,D",   0, SIZE_NONE,   0 },
  { "subu",    0x0880, 0x0760,             "z S,R,r", 0, SIZE_NONE,   0 },
  { "svc",     0x0530+CC_VC*0x1000,
               0x0AC0+(0xf-CC_VC)*0x1000,  "r",       0, SIZE_NONE,   0 },
  { "svs",     0x0530+CC_VS*0x1000,
               0x0AC0+(0xf-CC_VS)*0x1000,  "r",       0, SIZE_NONE,   0 },
 /* "swapn" is the same as NOT, but the swapn mnmonic is v8-higher only. */
  { "swapn",   0x8770, 0x7880,             "r",       0, SIZE_NONE,   cris_ver_v8p },
  { "swapw",   0x4770, 0xb880,             "r",       0, SIZE_NONE,   cris_ver_v8p },
  { "swapnw",  0xc770, 0x3880,             "r",       0, SIZE_NONE,   cris_ver_v8p },
  { "swapb",   0x2770, 0xd880,             "r",       0, SIZE_NONE,   cris_ver_v8p },
  { "swapnb",  0xA770, 0x5880,             "r",       0, SIZE_NONE,   cris_ver_v8p },
  { "swapwb",  0x6770, 0x9880,             "r",       0, SIZE_NONE,   cris_ver_v8p },
  { "swapnwb", 0xE770, 0x1880,             "r",       0, SIZE_NONE,   cris_ver_v8p },
  { "swapr",   0x1770, 0xe880,             "r",       0, SIZE_NONE,   cris_ver_v8p },
  { "swapnr",  0x9770, 0x6880,             "r",       0, SIZE_NONE,   cris_ver_v8p },
  { "swapwr",  0x5770, 0xa880,             "r",       0, SIZE_NONE,   cris_ver_v8p },
  { "swapnwr", 0xd770, 0x2880,             "r",       0, SIZE_NONE,   cris_ver_v8p },
  { "swapbr",  0x3770, 0xc880,             "r",       0, SIZE_NONE,   cris_ver_v8p },
  { "swapnbr", 0xb770, 0x4880,             "r",       0, SIZE_NONE,   cris_ver_v8p },
  { "swapwbr", 0x7770, 0x8880,             "r",       0, SIZE_NONE,   cris_ver_v8p },
  { "swapnwbr", 0xf770, 0x0880,             "r",       0, SIZE_NONE,  cris_ver_v8p },

  { "test",    0x0640, 0x0980,             "m D",     0, SIZE_NONE,   0 },
  { "test",    0x0b80, 0xf040,             "m s",     0, SIZE_FIELD,  0 },
  { "test",    0x0b80, 0xf040,             "m S",     0, SIZE_NONE,   0 },
  { "xor",     0x07B0, 0x0840,             "r,R",     0, SIZE_NONE,   0 },
  { NULL,      0,      0,		   0,	      0, 0,   0 } /* Terminate. */
};

/* End of cris-opc.c */
